// <!!-*-C++-*- file: binstream.h --->
// <!!-------------------------------------------------------------------------->
// <!! Copyright (C) 1996 Dietmar Kuehl >
// <!!   Universitaet Konstanz, Lehrstuhl fuer praktische Informatik I >
// <!!>
// <!! Permission to use, copy, modify, distribute and sell this >
// <!! software for any purpose is hereby granted without fee, provided >
// <!! that the above copyright notice appears in all copies and that >
// <!! both that copyright notice and this permission notice appear in >
// <!! supporting documentation. Dietmar Kuehl makes no representations
// <!! about the suitability of this software for any purpose. It is
// <!! provided "as is" without express or implied warranty.
// <!!-------------------------------------------------------------------------->

// Author: Dietmar Khl dietmar.kuehl@@uni-konstanz.de www.informatik.uni-konstanz.de/~kuehl
// Title:  A pair of streams for binary input/output to a file

// WORK TO DO:
// - support strings (just omitted because it is a little bit more complex)
// - maybe support for other structures like 'vector' can be added...
// - do real testing (currently this code should be considered to be untested...)

//------------------------------------------------------------------------------

#if !defined(BINSTREAM_H)
#define BINSTREAM_H

//------------------------------------------------------------------------------

#include <cstdlib>
#include <streambuf>

// It is sufficient to know that this is a type...

struct XDR;

//------------------------------------------------------------------------------

class binios
{
private:
  std::streambuf* sbuf;
  XDR*            xdrs;
  std::size_t     len; // used by the extractor for 'char*' similar to
		       // 'width' of 'ios', hence the naming of the
		       // corresponding functions below

  binios(binios const &);             // copying is not supported
  binios &operator= (binios const &); // assignment is not supported

public:
  enum operation
  {
    encode,
    decode
  };

  binios(std::streambuf *sb, operation op);
  virtual ~binios();

  std::streambuf *rdbuf() const { return sbuf; }
  void           rdbuf(std::streambuf *sb);

  // I decided that it makes not much sense to have this stream
  // 'protected'. It may be useful to directly encode data-types using
  // the XDR stream.
  XDR *xdrstream() { return xdrs; }

  std::size_t width() const { return len; }
  std::size_t width(size_t s);
};

//------------------------------------------------------------------------------

class obinstream: public binios
{
private:
  obinstream(obinstream const &);             // copying is not supported
  obinstream &operator= (obinstream const &); // assignment is not supported

public:
  obinstream(std::streambuf *sb);
  ~obinstream();

  obinstream &operator<< (bool &b);
  obinstream &operator<< (char &c); // NOTE: uses 4 bytes for a character!
  obinstream &operator<< (unsigned char &c);
  obinstream &operator<< (signed char &c);

  obinstream &operator<< (double &d);
  obinstream &operator<< (float &f);

  obinstream &operator<< (int &i);
  obinstream &operator<< (long &l);
  obinstream &operator<< (short &s);

  obinstream &operator<< (unsigned int &i);
  obinstream &operator<< (unsigned long &l);
  obinstream &operator<< (unsigned short &s);

  obinstream &operator<< (char const *str);
};

class ofbinstream: public obinstream
{
private:
  ofbinstream(ofbinstream const &);             // copying is not supported
  ofbinstream &operator= (ofbinstream const &); // assignment is not supported
public:
  ofbinstream(char const *fname);
  ~ofbinstream();
};

//------------------------------------------------------------------------------

class ibinstream: public binios
{
private:
  ibinstream(ibinstream const &);             // copying is not supported
  ibinstream &operator= (ibinstream const &); // assignment is not supported

public:
  ibinstream(std::streambuf *sb);
  ~ibinstream();

  ibinstream &operator>> (bool &b);
  ibinstream &operator>> (char &c); // NOTE: uses 4 bytes for a character!
  ibinstream &operator>> (unsigned char &c);
  ibinstream &operator>> (signed char &c);

  ibinstream &operator>> (double &d);
  ibinstream &operator>> (float &f);

  ibinstream &operator>> (int &i);
  ibinstream &operator>> (long &l);
  ibinstream &operator>> (short &s);

  ibinstream &operator>> (unsigned int &i);
  ibinstream &operator>> (unsigned long &l);
  ibinstream &operator>> (unsigned short &s);

  ibinstream &operator>> (char *str); // see the 'width()' member function.
};

class ifbinstream: public ibinstream
{
private:
  ifbinstream(ifbinstream const &);             // copying is not supported
  ifbinstream &operator= (ifbinstream const &); // assignment is not supported
public:
  ifbinstream(char const *fname);
  ~ifbinstream();
};

//------------------------------------------------------------------------------

#endif /* BINSTREAM_H */
